const { app, BrowserWindow } = require('electron');
const log = require('electron-log');
const uiLog = log.scope('ui');

const gotTheLock = app.requestSingleInstanceLock();

if (!gotTheLock) {
  app.quit();
  return;
}

var data = {
    mainWindow: null,
    deviceModuleChildProcess: null,
    deviceModuleNeedRestart: true,
    deviceModuleCommand: '-d32 -Dfile.encoding=UTF-8 -Djava.library.path=./drivers/ -splash:lib/screen.png -cp ".;ytc-client.jar" ru.ytimes.client.pc.main.Main'
};

function open(url) {
    require('electron').shell.openExternal(url);
}

function createWindow (needShow) {
  // Создаем окно браузера.
  const win = new BrowserWindow({
    title: 'YTimes Касса',
    width: 800,
    height: 600,
    webPreferences: {
      'node-integration': false,
      nodeIntegration: false
    },
    frame: false,
    show: false
  });

  win.webContents.on('console-message', (event, level, message, line, sourceId) => {
    uiLog.info(message);
  });

//  win.setFullScreen(true);
//  win.setSimpleFullScreen(true);
  win.loadURL("https://front.ytimes.ru/");

  win.webContents.on('new-window', function(event, url){
    event.preventDefault();
    open(url);
  });

  data.mainWindow = win;

  if (needShow) {
    win.maximize();
    win.show();
  }
}

function getFullAppPath() {
    return app.getAppPath() + "/../../../../";
}

/**
 * Модуль устройств запускается как дочерний процесс
 * И если он упадет, то автоматически перезапускается
 */

function runDeviceModule() {
  try {
    log.info("RUN update");
    require('child_process').execFileSync(getFullAppPath() + "tools/update/update.bat");
  }
  catch(error) {
    log.error("ERROR: " + error)
  }

  log.info("RUN: javaw.exe " + data['deviceModuleCommand']);

  var child = require('child_process').spawn(
    "javaw.exe",
    data['deviceModuleCommand'].split(' '),
    {
      cwd: getFullAppPath()
    }
  );
  data['deviceModuleChildProcess'] = child;

  child.stdout.on("data", (line) => {
    if (line && line.indexOf('START READY FOR ELECTRON') > -1) {
      data.mainWindow.maximize();
      data.mainWindow.show();
    }
    else if (line && line.indexOf('EXIT FOR ELECTRON') > -1) {
      app.quit();
    }
    else if (line && line.indexOf('RESTART FOR ELECTRON') > -1) {
      app.relaunch();
      app.quit();
    }
  });

  child.stderr.on("data", (err) => {
    log.log("Error child: " + err);
  });

  child.on("exit", (code) => {
    log.log('Exit child code: ' + code);
    if (code > 0 && data['deviceModuleNeedRestart']) {
        setTimeout(runDeviceModule, 1000);
    }
  });
}

//Запускаем само приложение
app.whenReady()
    .then(() => {
        log.transports.file.format = "{text}";
        log.transports.file.fileName = "front.log";
        log.transports.file.maxSize = 10485760;   //10mb
        log.transports.file.resolvePath = function() {
          return getFullAppPath() + 'logs/front.log';
        };
    })
    .then(() => {
        createWindow(false);
    })
    .then(() => {
        runDeviceModule();
    });

// Если все окошки закрыли, приложение нужно закрыть. Кроме Mac OS
app.on('window-all-closed', () => {
  // Для приложений и строки меню в macOS является обычным делом оставаться
  // активными до тех пор, пока пользователь не выйдет окончательно используя Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', () => {
   // На MacOS обычно пересоздают окно в приложении,
   // после того, как на иконку в доке нажали и других открытых окон нету.
  if (BrowserWindow.getAllWindows().length === 0 && process.platform == 'darwin') {
    createWindow(true)
  }
})

// SSL/TSL: this is the self signed certificate support
app.on('certificate-error', (event, webContents, url, error, certificate, callback) => {
  //log.info(certificate);
  // On certificate error we disable default behaviour (stop loading the page)
  // and we then say "it is all fine - true" to the callback
  event.preventDefault();
  callback(true);
});

/**
 * Выключаем дочерний процесс при выходе из программы
 */

var cleanExit = function() {
    app.quit();
};

app.on('SIGINT', cleanExit); // catch ctrl-c
app.on('SIGTERM', cleanExit); // catch kill

app.on('quit', function() {
  if (data['deviceModuleChildProcess'] && data['deviceModuleChildProcess'].pid) {
  log.info("Exit, try kill child processes: " + data['deviceModuleChildProcess'].pid);
    data['deviceModuleNeedRestart'] = false;
    process.kill(data['deviceModuleChildProcess'].pid, 'SIGTERM');
  }
});
